
;-------------------------------------------------------------------------------

Function set_checkpoint(C.Checkpoint, entity)
	; Delete any previous checkpoint instance.

	If C\entity <> 0 Then
		FreeEntity C\entity
	End If
	
	; Intanciate and configure the new checkpoint entity.
	
	C\entity = CopyEntity(entity)
	
	ShowEntity     C\entity
	ScaleEntity    C\entity, course_radius, course_radius, course_radius
	RotateEntity   C\entity, 0, C\angle + 180, 0
	PositionEntity C\entity, C\X, C\Y, C\Z
	EntityType     C\entity, COLLISION_COURSE
	EntityRadius   C\entity, course_radius * 0.75
	EntityFX       C\entity, 1
End Function

Function create_checkpoint()
	C.Checkpoint = New Checkpoint
	
	C\Z      = (entity_X + 4096) / 8
	C\Y      =  entity_Z         / 8
	C\X      = (4096 - entity_Y) / 8
	C\angle  = entity_angle
	C\name   = entity_name
	C\dest   = entity_dest
	C\entity = 0
	; i'm creating then destroying this sprite just to make sure one exists, because
	; i can't come up with a better way to verify that this checkpoint has a valid
	; radarPoint sprite before calling FreeEntity on it.
	;C\radarPoint = CreateSprite()

	set_checkpoint(C, ring_dead)
End Function

Function create_health(value#)
	If Not freeflight Then
		H.Health = New Health
		
		H\Z     = (entity_X + 4096) / 8
		H\Y     =  entity_Z         / 8
		H\X     = (4096 - entity_Y) / 8
		H\value = value
		H\halo  = CreateSprite()
	
		If      H\value >= 0.50 Then
			H\entity  = CopyEntity(food_L)
			H\balloon = CopyEntity(balloon_L)
		Else If H\value >= 0.25 Then
			H\entity  = CopyEntity(food_M)
			H\balloon = CopyEntity(balloon_M)
		Else
			H\entity  = CopyEntity(food_S)
			H\balloon = CopyEntity(balloon_S)
		End If
	
		ShowEntity     H\balloon
		PositionEntity H\balloon, H\X, H\Y, H\Z
			
		ShowEntity     H\entity
		PositionEntity H\entity, H\X, H\Y, H\Z
		EntityType     H\entity, COLLISION_HEALTH
		EntityRadius   H\entity, health_radius
		
		texture = LoadTexture("data\halo.png", 2)
		EntityTexture  H\halo, texture
		ScaleSprite    H\halo, health_radius * 2, health_radius * 2
		PositionEntity H\halo, H\X, H\Y, H\Z
		FreeTexture    texture
	End If	
End Function

;-------------------------------------------------------------------------------

Function entity_origin(val$)
	p1 = Instr(val, " ",      1)
	p2 = Instr(val, " ", p1 + 1)
	
	entity_X = Mid(val,      1, p1)
	entity_Y = Mid(val, p1 + 1, p2 - p1 - 1)
	entity_Z = Mid(val, p2 + 1, Len(val) - p2)
End Function

Function entity_begin()
	entity_type  = ""
	entity_name  = "noname"
	entity_dest  = ""
	entity_angle = 0
End Function

Function entity_item(key$, val$)
	If      key = "classname"  Then
		entity_type  = val
	Else If key = "targetname" Then
		entity_name  = val
	Else If key = "target"     Then
		entity_dest  = val
	Else If key = "angle"      Then
		entity_angle = val
	Else If key = "origin"     Then
		entity_origin(val)
	End If
End Function

Function entity_end()
	If      entity_type = "path_corner"       Then
		create_checkpoint()
	Else If entity_type = "item_health_small" Then
		create_health(0.10)
	Else If entity_type = "item_health_large" Then
		create_health(0.25)
	Else If entity_type = "item_health_mega"  Then
		create_health(0.50)
	End If
End Function

;-------------------------------------------------------------------------------

Function course_create(filename$)
	infile = ReadFile(filename)
	
	; Loop through the map file parsing lines and creating entities as needed.

	If infile <> 0 Then
		While Not Eof(infile)
			inline$ = ReadLine(infile)
			
			If Left$(inline, 1) = Chr$(34) Then
				k1 = Instr(inline, Chr$(34),      1)
				k2 = Instr(inline, Chr$(34), k1 + 1)
				v1 = Instr(inline, Chr$(34), k2 + 1)
				v2 = Instr(inline, Chr$(34), v1 + 1)
				
				key$ = Mid(inline, k1 + 1, k2 - k1 - 1)
				val$ = Mid(inline, v1 + 1, v2 - v1 - 1)
				
				entity_item(key, val)
				
			Else If Left$(inline, 1) = "{" Then
				entity_begin()
			Else If Left$(inline, 1) = "}" Then
				entity_end()
				
			End If
		Wend
		CloseFile(infile)
	End If

	; Sort checkpoints so that each immediately precedes its destination.

	For A.Checkpoint = Each Checkpoint
		For B.Checkpoint = Each Checkpoint
			If A\dest = B\name Then
				Insert A Before B
			End If
		Next
	Next
	
	; Mark the last checkpoint as a goal.
	
	set_checkpoint(Last Checkpoint, goal_dead)
	
	ShowEntity glow_fore_large
	ShowEntity glow_fore_small
	ShowEntity glow_back_large
	ShowEntity glow_back_small
	
End Function

Function course_delete()

	; Free any uneaten food.
	
	For H.Health = Each Health
		FreeEntity H\balloon
		FreeEntity H\entity
		FreeEntity H\halo
		Delete H
	Next
	
	; Free any unclaimed checkpoints
	
	For C.Checkpoint = Each Checkpoint
		FreeEntity C\entity
		; i guess maybe this doesn't exist at some point that I don't know of.
		;FreeEntity C\radarPoint
		Delete C
	Next
	
	HideEntity glow_fore_large
	HideEntity glow_fore_small
	HideEntity glow_back_large
	HideEntity glow_back_small
	
End Function

Function course_update()
	k# = (1.0 + Sin(360.0 * MilliSecs() / 1000.0) * 0.25) * health_radius * 2.0

	; Spin the food and pulse the food halos.

	For H.Health = Each Health
		TurnEntity  H\entity, 0, 5, 0
		ScaleSprite H\halo, k, k
	Next

	; Spin the checkpoint glow sprites.

	TurnEntity glow_fore_large, 0, 0,  1
	TurnEntity glow_fore_small, 0, 0, -1
	TurnEntity glow_back_large, 0, 0, -1
	TurnEntity glow_back_small, 0, 0,  1
End Function

;-------------------------------------------------------------------------------

; The player has grabbed food.  Remove the food and credit the player's health.

Function course_food()
	P.Player = First Player

	For H.Health = Each Health
		If H\entity = EntityCollided(P\entity, COLLISION_HEALTH) Then
			P\health = P\health + H\value
			
			If P\health > 1.0 Then P\health = 1.0
			
			FreeEntity H\halo
			FreeEntity H\entity
			FreeEntity H\balloon
			Delete H
		EndIf
	Next
	
	FreeEntity entity
End Function

; The player has hit the next checkpoint.  Remove it.

Function course_ring()

	If First Checkpoint <> Null Then
		C.Checkpoint = First Checkpoint
		
		FreeEntity C\entity
		FreeEntity C\radarPoint
		Delete First Checkpoint
	End If
	
	; Hilite the next checkpoint.
	
	If First Checkpoint <> Null Then
		C.Checkpoint = First Checkpoint
		
		; Determine whether the next checkpoint is the goal.
		
		If First Checkpoint = Last Checkpoint Then
			set_checkpoint(C, goal_live)
		Else
			set_checkpoint(C, ring_live)
		End If
		
		; Position the halo at the next checkpoint.
		
		PositionEntity glow_fore_large, C\X, C\Y, C\Z
		PositionEntity glow_fore_small, C\X, C\Y, C\Z
		PositionEntity glow_back_large, C\X, C\Y, C\Z
		PositionEntity glow_back_small, C\X, C\Y, C\Z
		
		RotateEntity   glow_fore_large, 0, C\angle,       0
		RotateEntity   glow_fore_small, 0, C\angle,       0
		RotateEntity   glow_back_large, 0, C\angle + 180, 0
		RotateEntity   glow_back_small, 0, C\angle + 180, 0
	End If
End Function

;-------------------------------------------------------------------------------