
;-------------------------------------------------------------------------------

Function enter_start()
	JoyHit(1)
	KeyHit(57)
	
	; Initialize the course and player
	; Radar implementation might break if the checkpoints are not created before
	; the player is.
	C.Checkpoint = First Checkpoint
	player_create(C\X, C\Y, C\Z, C\angle)

	; The first checkpoint is the start, not a ring.  Skip it.
	
	course_ring()

	P.Player = First Player
	
	; Load the get-ready image.
	
	overlay = LoadSprite("data\ready.png", 2)
	EntityParent   overlay, P\camera
	ScaleSprite    overlay, 7.5, 7.5
	PositionEntity overlay, 0, 0, 10
End Function

Function leave_start()
	EntityParent overlay, 0
	FreeEntity   overlay
End Function

Function update_start()
	P.Player = First Player
	
	PositionEntity(P\entity, P\X, P\Y, P\Z)

	fade_in()
	camera_update()
	course_update()
	shadow_update()
	
	; If the joystick button is pressed, start the level with joystick control.
	
	If JoyHit(1) Then
		P\control_mode = CONTROL_JOYSTICK
		Return STATE_LEVEL
	End If	
	
	; If the space bar is pressed, start the level with keyboard control.
	
	If KeyHit(57) Then
		P\control_mode = CONTROL_KEYBOARD
		Return STATE_LEVEL
	End If

	; Allow the player and overlay to be hidden for clean screenshots.

	If KeyHit(65) Then
		HideEntity(P\entity)
		HideEntity(P\propeller)
		HideEntity(overlay)
	End If
	If KeyHit(66) Then
		ShowEntity(P\entity)
		ShowEntity(P\propeller)
		ShowEntity(overlay)
	End If
	
	Return STATE_START
End Function

;-------------------------------------------------------------------------------

Function enter_level()
End Function

Function leave_level()
End Function

Function update_level()
	P.Player = First Player

	; Handle the player and course.

	new_state = player_update()
	course_update()
	shadow_update()
	camera_update()
	particle_update()
	fade_in()

	Return new_state
End Function

;-------------------------------------------------------------------------------

Function enter_goal()
	P.Player = First Player
	
	JoyHit(1)
	KeyHit(57)

	; Load the goal image.
	
	overlay = LoadSprite("data\goal.png", 2)
	EntityParent   overlay, P\camera
	ScaleSprite    overlay, 7.5, 7.5
	PositionEntity overlay, 0, 0, 10
End Function

Function leave_goal()
	player_delete()
	course_delete()
End Function

Function update_goal()
	camera_update()	
	course_update()
	
	; If a button or key is hit, fade and return to the title screen.
	
	If JoyHit(1) Or KeyHit(57) Then
		Return STATE_FADEOUT
	Else
		Return STATE_GOAL
	End If
End Function

;-------------------------------------------------------------------------------

Global count
Global initial

Function enter_high()
	P.Player = First Player
	
	JoyHit(1)
	KeyHit(57)

	count = 1
	high_time(selection) = P\lasttime - P\starttime

	; Load the high-score image.
	
	overlay = LoadSprite("data\high.png", 2)
	EntityParent   overlay, P\camera
	ScaleSprite    overlay, 3.0, 3.0
	PositionEntity overlay, 0, -0.5, 3
	
	initial = create_initial(high_name(selection), count)
	PositionEntity initial, 0, -1, 2
End Function

Function leave_high()
	FreeEntity initial
	player_delete()
	course_delete()
	high_write()
End Function

Function update_high()
	camera_update()	
	course_update()

	; If the user types a letter, add it to the name.

	key = GetKey()

	If Asc("a") <= key And key <= Asc("z") And count < 4 Then
		A$ = Mid(high_name(selection), 1, 1)
		B$ = Mid(high_name(selection), 2, 1)
		C$ = Mid(high_name(selection), 3, 1)
		K$ = Upper(Chr(key))
		
		Select count
			Case 1 high_name$(selection) = K + B + C
			Case 2 high_name$(selection) = A + K + C
			Case 3 high_name$(selection) = A + B + K
		End Select			
		
		count = count + 1
		
		FreeEntity     initial
		initial = create_initial(high_name(selection), count)
		PositionEntity initial, 0, -1, 2
	End If

	; If the user hits backspace, then backspace.
	
	If KeyHit(14) And count > 1 Then
		count = count - 1
		FreeEntity     initial
		initial = create_initial(high_name(selection), count)
		PositionEntity initial, 0, -1, 2
	End If
	
	; If the button or spacebar is hit, fade and return to the title screen.
	
	If JoyHit(1) Or KeyHit(57) Or KeyHit(28) Then
		Return STATE_FADEOUT
	Else
		Return STATE_HIGH
	End If
End Function

;-------------------------------------------------------------------------------